from os.path import join


class Settings:
    def __init__(self,
                 language,
                 plugins_base_path='/opt',
                 grpc_version="1.54.3",
                 protobuf_version="3.21.12",
                 DATA_DIR='/data',
                 CORE_DIR='/protoc-polyglot'):
        self.language = language

        self.plugins_base_path = plugins_base_path
        self.grpc_version = grpc_version
        self.protobuf_version = protobuf_version

        self.plugin_path_doc = f'{plugins_base_path}/doc/protoc-gen-doc'

        self.protobuf_folder = 'protobuf'
        self.grpc_folder = 'grpc'
        self.languages_that_have_external_plugin = ['go', 'java', 'js', 'rust']

        self.DATA_DIR = DATA_DIR
        self.CORE_DIR = CORE_DIR
        self.ROOT_PROTOS = join(DATA_DIR, 'protos')
        self.OUTPUT_ROOT = join(DATA_DIR, 'output')
        self.DOC_OUTPUT_DIR = join(self.OUTPUT_ROOT, 'doc')

        self.services_yaml = join(self.ROOT_PROTOS, 'services.yml')



#!/usr/bin/python3
import os
import yaml
from os.path import dirname, join
from zipfile import ZipFile
import requests


class Tools:
    @staticmethod
    def get_service_info(services_yaml, name: str, key: str = 'files') -> dict[str, list[str]]:
        with open(services_yaml, 'r') as file:
            data = yaml.safe_load(file)
            data = data['services']

            if name == "":
                return {n: data[n][key] for n in data}
            else:
                return {name: data[name][key]}

    @staticmethod
    def get_files_from_directory(dir: str):
        proto_files = []
        for root, directories, files in os.walk(dir):
            for filename in files:
                if ".proto" in filename:
                    proto_files.append(join(root, filename))
        return proto_files

    @staticmethod
    def zip_directory(dir: str):
        zip_file = f'{dir}.zip'
        with ZipFile(zip_file, 'w') as zip:
            for root, directories, files in os.walk(dir):
                for filename in files:
                    filepath = join(root, filename)
                    zip.write(filepath, filepath[len(dir):])
        return zip_file

    @staticmethod
    def download_file(name: str, saveto):
        url = f"https://github.com/quantasoftcz/protoc-polyglot/files/{name}"
        response = requests.get(url)

        if response.status_code == 200:
            os.makedirs(dirname(saveto), exist_ok=True)
            with open(saveto, 'wb') as f:
                f.write(response.content)
            return True
        else:
            print(f"Failed to download {url}. Status code: {response.status_code}")
            return False



#!/usr/bin/python3
import os, sys, shutil, yaml
from os.path import dirname, basename, abspath, join
from fire import Fire
from zipfile import ZipFile
import requests

from protoc_polyglot.tools import Tools
from protoc_polyglot.settings import *


class CommonInterface:
    def __init__(self, settings: Settings=None):
        self.settings = settings

    def get_plugin_executable_path(self):
        return join(self.get_plugin_dir(), self.plugin_name)

    def get_plugin_dir(self) -> str:
        return join(self.settings.plugins_base_path, self.get_language())

    def get_protobuf_path(self) -> str:
        return join(self.settings.plugins_base_path, self.settings.protobuf_folder)
    
    def get_grpc_path(self) -> str:
        return join(self.settings.plugins_base_path, self.settings.grpc_folder)

    def get_language(self):
        assert hasattr(self, 'plugin_name'), "This function can only be called on LanguageInterface"

        return self.settings.language

    def download_grpc_and_protobuf(self):
        if os.path.exists(self.get_grpc_path()) and os.path.exists(self.get_protobuf_path()):
            return True

        print("gRPC or Protobuf missing, downloading...")
        name = f"grpc-{self.settings.grpc_version}_protobuf-{self.settings.protobuf_version}.zip"
        return Tools.download_file(name, self.get_grpc_path())

    def download_plugin(self):
        assert hasattr(self, 'plugin_name'), "This function can only be called on LanguageInterface"

        if self.get_language() not in self.settings.languages_that_have_external_plugin or os.path.exists(self.get_plugin_dir()):
            return True

        print("Plugin missing, downloading...")
        return Tools.download_file(self.plugin_name, self.get_plugin_dir())

    def list(self):
        print(os.getcwd())
        
        with os.scandir(self.settings.CORE_DIR) as entries:
            print('Supported languages:')
            folders = [entry.name for entry in entries if entry.is_dir() and entry.name not in ['__pycache__']]
            for f in folders:
                print(f)
        
        with open(self.settings.services_yaml, 'r') as file:
            data = yaml.safe_load(file)
            print('Found services:')
            for name in data:
                print(f'- {name}')
                for file in data[name]['files']:
                    print(f'\t- {file}')

    def make(self, dir: str):
        if self.__class__.__name__ == "CommonInterface":
            print("ERROR: Use a language")
            exit(1)

        dir_protos = os.path.abspath(dir)
        files = Tools.get_files_from_directory(dir_protos)
        dir_output = join(dir_protos, "output")
        self._compile(dir_protos, dir_output, files)
        return Tools.zip_directory(dir_output)
    
    def protoc(self, name:str=""):
        assert hasattr(self, 'plugin_name'), "This function can only be called on LanguageInterface"

        if not self.download_grpc_and_protobuf():
            raise RuntimeError("Could not find gRPC and Protobuf package")
        if not self.download_plugin():
            raise RuntimeError(f"Could not find {name} release")
        
        if self.__class__.__name__ == "CommonInterface":
            print("ERROR: Use a language")
            exit(1)
        
        files_dict = Tools.get_service_info(self.settings.services_yaml, name)
        for name, files in files_dict.items():
            self._compile(self.settings.ROOT_PROTOS, self._get_dir_output(name), files)

        self.doc()
        
    def _get_dir_output(self, name:str):
        return join(self.settings.OUTPUT_ROOT, self.get_language(), name)

    def doc(self, md:bool=True, html:bool=False):
        for name, files in Tools.get_service_info(self.settings.services_yaml, name='').items():
            aux = ' '.join(files)
            os.makedirs(self.settings.DOC_OUTPUT_DIR, exist_ok=True)
            if md:
                com = f'protoc -I {self.settings.ROOT_PROTOS} --plugin=protoc-gen-doc={self.settings.plugin_path_doc} --doc_out={self.settings.DOC_OUTPUT_DIR} --doc_opt=markdown,{name}.md {aux}'
                print(com)
                os.system(com)

                # fix links by "a name" -> "a id"
                text = open(join(self.settings.DOC_OUTPUT_DIR, f'{name}.md'), 'r', encoding="utf-8").read()
                text = text.replace('<a name=', '<a id=')
                text = text.replace(' &lt;br&gt;', ' <br>')
                open(join(self.settings.DOC_OUTPUT_DIR, f'{name}.md'), 'w', encoding="utf-8").write(text)
            if html:
                com = f'protoc -I {self.settings.ROOT_PROTOS} --plugin=protoc-gen-doc={self.settings.plugin_path_doc} --doc_out={self.settings.DOC_OUTPUT_DIR} --doc_opt=html,{name}.html {aux}'
                print(com)
                os.system(com)
        print(f'\ndone, docs saved in {self.settings.DOC_OUTPUT_DIR}')



#!/usr/bin/python3
import os, sys
sys.path.insert(0, os.path.abspath('..'))
from protoc_polyglot.cli import *


class LanguageInterface(CommonInterface):
    plugin_name = 'protoc-gen-go'

    def _compile(self, dir_protos:str, dir_output: str, files:list[str]) -> None:
        shutil.rmtree(dir_output, ignore_errors=True)
        os.makedirs(dir_output, exist_ok=False)

        com = f"""/usr/bin/protoc \
        -I {ROOT_PROTOS} \
        --plugin=protoc-gen-go={self.get_plugin_executable_path()} \
        --go_out={dir_output} \
        --go_opt=paths=source_relative \
        --go-grpc_out={dir_output} \
        --go-grpc_opt=paths=source_relative \
        {" ".join(files)}"""
        
        print(com)
        os.system(com)



#!/usr/bin/python3
from fire import Fire
import importlib
import sys
import os
from os.path import abspath
from os.path import dirname
import argparse
from enum import Enum


class Run_mode(Enum):
    DOCKER = 1
    PYPI = 2

# import of core is different based on whether we run in PyPi package or in Docker
run_mode = Run_mode.DOCKER if os.path.abspath(__file__) == '/protoc_polyglot/cli.py' else Run_mode.PYPI

if run_mode == Run_mode.DOCKER:
    sys.path.append('/')

from protoc_polyglot.common_interface import *

def main():
    parser = argparse.ArgumentParser(description="Process protoc-polyglot command-line arguments.")

    parser.add_argument("-l", "--languages", help="Programming languages for protoc generation.", nargs='+', required=True)
    parser.add_argument("-y", "--service-yml", help="Service YAML file, directory or specific file.", required=True)
    parser.add_argument("-n", "--service-name", help="Service name.", default="")
    parser.add_argument("-d", "--directory-input", help="Directory input.")
    parser.add_argument("-f", "--files", nargs='+', help="List of files to process.")
    parser.add_argument("-o", "--output-dir", default="output", help="Output directory (default: output).")

    args = parser.parse_args()

    print(f"Languages: {args.languages}")
    print(f"Services YML: {args.service_yml}")
    if args.service_name:
        print(f"Service Name: {args.service_name}")
    if args.directory_input:
        print(f"Directory Input: {args.directory_input}")
    if args.files:
        print(f"Files: {', '.join(args.files)}")
    print(f"Output Directory: {args.output_dir}")

    for language in args.languages:
        try:
            function = 'protoc'

            if run_mode == Run_mode.PYPI:
                setup_module = importlib.import_module('protoc_polyglot.' + language + '.cli', package="protoc-polyglot")
                LanguageInterface = getattr(setup_module, 'LanguageInterface')
                settings = Settings('plugins', DATA_DIR='', CORE_DIR=dirname(abspath(__file__)))
                LanguageInterface = LanguageInterface(settings)
            else:
                module_dir = os.path.join(os.path.dirname(__file__), language)

                sys.path.insert(0, module_dir)

                module_path = os.path.join(module_dir, 'language_interface.py')

                spec = importlib.util.spec_from_file_location('cli', module_path)
                setup_module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(setup_module)

                LanguageInterface = getattr(setup_module, 'LanguageInterface')
                settings = Settings(language)
                LanguageInterface = LanguageInterface(settings)

                sys.path.pop(0)

            if hasattr(LanguageInterface, function):
                fc = getattr(LanguageInterface, function)
                fc(args.service_name)
            else:
                print(f"Function '{function}' not found in '{language}.cli'")
        except ModuleNotFoundError:
            print(f"Language '{language}' not found.")
        # except Exception as e:
        #     print(f"An error occurred: {str(e)}")

if __name__ == '__main__':
    main()